package com.pigbite.cdh.acting;

/**
 * This action hooks into the multipart form processing offered by Cocoon and
 * uploads a file into the area specified in the <i>upload-dir</i> parameter.
 *
 * @author <a href="mailto:jez@pigbite.com">Jeremy Aston</a>
 * @version 1.00
 */

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.Constants;
import org.apache.cocoon.environment.Context;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Redirector;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.acting.AbstractConfigurableAction;

import java.io.*;
import java.util.*;
import java.io.BufferedInputStream;
import org.apache.cocoon.components.request.multipart.*;

public class FileUploadAction extends AbstractConfigurableAction implements ThreadSafe
{
	
	/** Name of the parameter that holds the upload directory name to use **/
	private static final String UPLOAD_FOLDER_PARAMETER = "upload-dir";
	
	/**
	 * Put an upload file into a specficed area
	 * Requires that the upload directory is specified as a parameter to the action called <i>upload-dir</i>
	 */
	public Map act (Redirector redirector, SourceResolver resolver, Map objectModel, String source, Parameters parameters)
		throws Exception
	{
        Request request = ObjectModelHelper.getRequest(objectModel);
        Context context = ObjectModelHelper.getContext(objectModel);
		byte[] buffer = new byte[4096];
		HashMap results = new HashMap();
    	
		if (request != null)
		{
			
			// Loop through the parameter names to find any file upload stuff
			getLogger().debug( "Request Parameters" );
     		for ( Enumeration e = request.getParameterNames(); e.hasMoreElements() ;)
			{
				
                String param = (String) e.nextElement();
				
                Object value = request.get( param );
			

				getLogger().debug( "[" + param + "] [" + value.getClass().getName() + "] : " + value );
                
                if ( value instanceof FilePart )
                {
					// Have got a File object (or sub class - hence the casting ) which can be worked on...
					// Now get the filename and the real path to form the file route.
					
					String filename = ( (FilePart)value ).getFileName();
					String uploadFolder = (String)parameters.getParameter( UPLOAD_FOLDER_PARAMETER );
					String realPath = context.getRealPath("/");
					
					// Do a sanity check on the upload folder
					
					if ( uploadFolder == null || uploadFolder.length() == 0 )
					{
						getLogger().debug(
							"Missing " + UPLOAD_FOLDER_PARAMETER + 
							" parameter or no upload folder specified in the " + 
							UPLOAD_FOLDER_PARAMETER + " parameter" );
						
						return null;
					}
					
					// Concatenate the realPath folder to the upload folder
					
					if ( realPath != null )
					{
						uploadFolder = realPath + uploadFolder;	   	     
					}
					
  					getLogger().debug("Uploading " + filename + " to " + uploadFolder );
	
					// Create the upload folder if needs be
					
   	     			File folder = new File( uploadFolder );
   	     			if ( !folder.exists() )
					{
   	     				folder.mkdir();
					}
   	     
					// Check to see if the object is a FilePartFile object as this can
					// simply be renamed.
					
					if ( value instanceof FilePartFile )
					{
						
          				getLogger().debug( "Renaming " + filename + " to " + uploadFolder + File.separator + filename );
						( (FilePartFile)value ).getFile().renameTo( new File( uploadFolder + File.separator + filename) );
						
					} 
					else
					{	
                
						getLogger().debug( "Streaming file to " + uploadFolder + File.separator + filename );
            
		   	     		FileOutputStream out = new FileOutputStream( uploadFolder + File.separator + filename );
   		     			InputStream in = ( (FilePart)value ).getInputStream();
   	     			
						int read = in.read( buffer );
   	     			
						while ( read > 0 )
						{
   	     					out.write( buffer, 0, read );
   	     					read = in.read( buffer );
   	     				}
   	     	
   	     				out.close();
					}
					
					request.setAttribute( "upload-file", filename );
			
				}
				else if ( value instanceof String )
				{
					getLogger().debug( "Skipping parameter: " + (String)value );
				}
				else
				{
              		getLogger().debug("something else");
        		}
			}
					

			String contentType = request.getContentType();
			getLogger().debug( "Content Type: " + contentType );

		}

		return Collections.unmodifiableMap(results);
		
    }

}
